'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var express = _interopDefault(require('express'));
var fetch = _interopDefault(require('node-fetch'));
var ICAL = _interopDefault(require('ical.js'));
var moment = _interopDefault(require('moment'));
var colorString = _interopDefault(require('color-string'));
var dotenv = _interopDefault(require('../.env.json'));

var icalMerger = { "prodid": "-//Jacob Mischka//iCal Merger//EN", "version": "2.0" };

function merge(inputs) {
	var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

	if (!Array.isArray(inputs)) inputs = [].concat(Array.prototype.slice.call(arguments));

	var calendar = void 0;
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = inputs[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var input = _step.value;

			try {
				var jcal = ICAL.parse(input);
				var cal = new ICAL.Component(jcal);

				if (!calendar) {
					calendar = cal;
					calendar.updatePropertyWithValue('prodid', icalMerger.prodid);
					calendar.updatePropertyWithValue('version', icalMerger.version);

					if (options.calname) calendar.updatePropertyWithValue('x-wr-calname', options.calname);
					if (options.timezone) calendar.updatePropertyWithValue('x-wr-timezone', options.timezone);
					if (options.caldesc) calendar.updatePropertyWithValue('x-wr-caldesc', options.caldesc);
				} else {
					var _iteratorNormalCompletion2 = true;
					var _didIteratorError2 = false;
					var _iteratorError2 = undefined;

					try {
						for (var _iterator2 = cal.getAllSubcomponents('vevent')[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
							var vevent = _step2.value;

							calendar.addSubcomponent(vevent);
						}
					} catch (err) {
						_didIteratorError2 = true;
						_iteratorError2 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion2 && _iterator2.return) {
								_iterator2.return();
							}
						} finally {
							if (_didIteratorError2) {
								throw _iteratorError2;
							}
						}
					}
				}
			} catch (e) {
				console.error('Failed to merge: ' + e + '\n\nWith input: ' + input);
			}
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	if (!calendar) {
		console.error('No icals parsed successfully');
		return;
	}

	return calendar.toString();
}

var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();

















































var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

function isCalendarVisible(calendar, keys) {
	if (!Array.isArray(keys)) keys = [keys];

	return calendar && (!calendar.private || Boolean(calendar.key) && typeof calendar.key === 'string' && keys.includes(calendar.key));
}









function getDeepCalendarIdsFromSubGroups(calendarGroup, allCalendars, allCalendarGroups) {
	var calendarIds = [];

	if (calendarGroup.subGroups && calendarGroup.subGroups.length > 0) {
		var _subGroups2 = calendarGroup.subGroups.map(function (id) {
			return allCalendarGroups[id];
		});
		var _iteratorNormalCompletion10 = true;
		var _didIteratorError10 = false;
		var _iteratorError10 = undefined;

		try {
			for (var _iterator10 = _subGroups2[Symbol.iterator](), _step10; !(_iteratorNormalCompletion10 = (_step10 = _iterator10.next()).done); _iteratorNormalCompletion10 = true) {
				var _subGroup = _step10.value;

				calendarIds.push.apply(calendarIds, toConsumableArray(getDeepCalendarIdsFromSubGroups(_subGroup, allCalendars, allCalendarGroups)));
			}
		} catch (err) {
			_didIteratorError10 = true;
			_iteratorError10 = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion10 && _iterator10.return) {
					_iterator10.return();
				}
			} finally {
				if (_didIteratorError10) {
					throw _iteratorError10;
				}
			}
		}
	}

	if (calendarGroup.calendars && calendarGroup.calendars.length > 0) {
		calendarIds.push.apply(calendarIds, toConsumableArray(calendarGroup.calendars));
	}

	return calendarIds;
}

/* eslint-env node */

var app = express();

app.use(express.static('public'));

app.get('/.env.json', function (req, res) {
	var options = {
		root: './',
		dotfiles: 'allow'
	};

	res.sendFile('.env.json', options);
});

app.get('/combine.ics', function (req, res) {
	if (!req.query.urls) {
		res.sendStatus(400);
		return;
	}

	var icals = getIcalsFromUrls(req.query.urls);

	setHeaders(res);

	var options = {};
	if (dotenv && dotenv.combine) options = Object.assign({}, dotenv.combine);
	options = Object.assign({}, options, req.query);

	icals.then(function (icals) {
		res.send(merge(icals, options));
	}).catch(function (err) {
		console.error('Error merging: ' + err);
	});
});

if (dotenv && dotenv.calendars) {
	for (var calendarName in dotenv.calendars) {
		var calendarConfig = dotenv.calendars[calendarName];

		respondWithCalendar(calendarConfig, calendarName);
	}

	if (dotenv.calendarGroups) {
		for (var _calendarName in dotenv.calendarGroups) {
			var _calendarConfig = dotenv.calendarGroups[_calendarName];

			respondWithCalendar(_calendarConfig, _calendarName);
		}
	}
}

app.get('/:calendarId', function (req, res) {
	res.sendFile('index.html', {
		root: './public/'
	});
});

function respondWithCalendar(calendar, calendarName) {
	app.get('/' + calendarName + '.ics', function (req, res) {
		var isVisible = function isVisible(calendar) {
			return isCalendarVisible(calendar, req.query.key);
		};

		if (!calendar || !isVisible(calendar)) {
			res.sendStatus(501);
			return;
		}

		var urls = [];
		if (calendar.url) urls.push(calendar.url);

		if (calendar.calendars || calendar.subGroups) {
			var calIds = getDeepCalendarIdsFromSubGroups(calendar, dotenv.calendars, dotenv.calendarGroups);

			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = calIds[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var calId = _step.value;

					var _calendar = dotenv.calendars[calId];
					if (isVisible(_calendar)) {
						if (_calendar.url) urls.push(_calendar.url);
						if (_calendar.subCalendars) urls = urls.concat(_calendar.subCalendars.filter(isVisible).map(function (subCal) {
							return subCal.url;
						}));
					}
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}
		}

		if (calendar.subCalendars) urls = urls.concat(calendar.subCalendars.filter(isVisible).map(function (subCal) {
			return subCal.url;
		}));

		urls = Array.from(new Set(urls));

		var icals = getIcalsFromUrls(urls);

		setHeaders(res);

		var options = Object.assign({}, calendar);

		icals.then(function (icals) {
			res.send(merge(icals, options));
		}).catch(function (err) {
			console.error('Error merging: ' + err);
		});
	});
}

function setHeaders(res) {
	res.set('Expires', 'Mon, 01 Jan 1990 00:00:00 GMT');
	res.set('Date', new Date().toGMTString());
	res.set('Content-Type', 'text/calendar; charset=UTF-8');
	res.set('Cache-Control', 'no-cache, no-store, max-age=0, must-revalidate');
	res.set('Pragma', 'no-cache');
}

var port = app.get('env') === 'production' ? 80 : process.env.PORT || 3000;

app.listen(port, function () {
	console.log('Listening on ' + port); // eslint-disable-line no-console
});

function getIcalsFromUrls(urls) {
	var promises = [];
	var _iteratorNormalCompletion2 = true;
	var _didIteratorError2 = false;
	var _iteratorError2 = undefined;

	try {
		var _loop = function _loop() {
			var url = _step2.value;

			promises.push(fetch(url).then(function (response) {
				return response.text();
			}).then(function (text) {
				return text;
			}).catch(function (err) {
				console.error('Error reading ' + url + ': ' + err);
				return err;
			}));
		};

		for (var _iterator2 = urls[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
			_loop();
		}
	} catch (err) {
		_didIteratorError2 = true;
		_iteratorError2 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion2 && _iterator2.return) {
				_iterator2.return();
			}
		} finally {
			if (_didIteratorError2) {
				throw _iteratorError2;
			}
		}
	}

	return Promise.all(promises);
}
